% The Linearized gain method
classdef ssfm2 < hmevolution
    methods
        function [v,hstiff,rerror] = solver_singlestep(self,v,h,fba)
            rerror = -1;
            hstiff = h;

            L = self.L_dispersion; K = self.L_gainsat;

            g1 = self.g0/(1+norm(v)^2/self.Esatw);
            g2 = -2*(g1^2/self.g0/self.Esatw)*real(dot(v,(L+g1*K).*v));

            self.method_information = max(self.method_information, abs(g2*h/4/g1));
            
            if bitand(self.method_variant,1)
                %v = v.*exp(L*h/2 + (g1+g1*g2*(h/4)/sqrt(g1^2+(g2*(h/4))^2))*(h/2)*K);
                v = v.*exp(L*h/2 + g1*(h/2)*K);
            else
                v = v.*exp(L*h/2 + (g1+g2*(h/4))*(h/2)*K);
            end

            % Nonlinear step
            if bitand(self.method_variant,4)
                v = fft(self.runge_kutta(h,ifft(v),fba));
            else
                v = fft(self.quadratic_exact(h,ifft(v),fba));
            end
                                    
            g1 = self.g0/(1+norm(v)^2/self.Esatw);
            %g2 = -2*(g1^2/self.g0/self.Esatw)*real(dot(v,(L+g1*K).*v));
            if bitand(self.method_variant,1)
                %v = v.*exp(L*h/2 + (g1+g1*g2*(h/4)/sqrt(g1^2+(g2*(h/4))^2))*(h/2)*K);
                v = v.*exp(L*h/2 + g1*(h/2)*K);
            else
                v = v.*exp(L*h/2 + (g1+g2*(h/4))*(h/2)*K);
            end
        end

        function vsol = runge_kutta(self,h,v,fba)
            abs_v_2 = abs(v).^2;
            k1 = 0.5*h*(self.deltagamma*abs_v_2 - self.sigma*abs_v_2.^2 - fba).*v;
            abs_v_2 = abs(v+k1).^2;
            vsol = v + h*(self.deltagamma*abs_v_2 - self.sigma*abs_v_2.^2 - fba).*(v+k1);
        end

        function vsol = quadratic_exact(self,h,v,fba)
            vsol = v.*exp(-self.deltagamma/(2*real(self.deltagamma))*log(1-2*real(self.deltagamma)*h*abs(v).^2));
        end

    end % Methods

end % Classdef
