% The Linearized gain method
classdef ssfm4 < hmevolution
    methods
        function v = singlestep_2o(self,v,h,fba)
            L = self.L_dispersion; K = self.L_gainsat;

            g1 = self.g0/(1+norm(v)^2/self.Esatw);
            dvdz = (L+g1*K).*v;
            g2 = -2*(g1^2/self.g0/self.Esatw)*real(dvdz'*v);

            if bitand(self.method_variant,1)
                v = v.*exp(L*h/2 + (g1+g1*g2*(h/4)/sqrt(g1^2+(g2*(h/4))^2))*(h/2)*K);
            else
                v = v.*exp(L*h/2 + (g1+g2*(h/4))*(h/2)*K);
            end

            % Nonlinear step
            if bitand(self.method_variant,4)
                v = fft(self.runge_kutta(h,ifft(v),fba));
            else
                v = fft(self.quadratic_exact(h,ifft(v),fba));
            end

            g1 = self.g0/(1+norm(v)^2/self.Esatw);
            if bitand(self.method_variant,8)
                g2 = -2*(g1^2/self.g0/self.Esatw)*real(dot(v,(L+g1*K).*v));
            end
            if bitand(self.method_variant,1)
                v = v.*exp(L*h/2 + (g1+g1*g2*(h/4)/sqrt(g1^2+(g2*(h/4))^2))*(h/2)*K);
            else
                v = v.*exp(L*h/2 + (g1+g2*(h/4))*(h/2)*K);
            end
        end
        
        function [v,hstiff,rerror] = solver_singlestep(self,v,h,fba)
%             rerror = -1;
            hstiff = h;
            v1 = self.singlestep_2o(v,h,fba);

            v2 = self.singlestep_2o(v,h/2,fba);
            v2 = self.singlestep_2o(v2,h/2,fba);

            v3 = self.singlestep_2o(v,h/4,fba);
            v3 = self.singlestep_2o(v3,h/4,fba);
            v3 = self.singlestep_2o(v3,h/4,fba);
            v3 = self.singlestep_2o(v3,h/4,fba);
            
            v = (32*v3-12*v2+v1)/21;
            rerror = norm(v-v3)/norm(v);
        end

        function vsol = runge_kutta(self,h,v,fba)
            abs_v_2 = abs(v).^2;
            k1 = 0.5*h*(self.deltagamma*abs_v_2 - self.sigma*abs_v_2.^2 - fba).*v;

            abs_v_2 = abs(v+k1).^2;
            vsol = v + h*(self.deltagamma*abs_v_2 - self.sigma*abs_v_2.^2 - fba).*(v+k1);
        end

        function vsol = quadratic_exact(self,h,v,~)
            vsol = v.*exp(-self.deltagamma/(2*real(self.deltagamma))*log(1-2*real(self.deltagamma)*h*abs(v).^2));
        end


        function err=test_N_step(self,v0,h)
            % Check the RK and exact formula match
            v1_rk = self.runge_kutta(h,v0,0);
            v1_qe = self.quadratic_exact(h,v0,0);
            
            err = norm(v1_rk-v1_qe)/norm(v0);
            fprintf('Error in N-step solve: %.5g\n',err);
        end

    end % Methods

end % Classdef
