% The Linearized gain method
classdef ssfm_s2 < hmevolution
    methods
        function v = singlestep_1o(self,v,h,fba)
            L = self.L_dispersion; K = self.L_gainsat;

            g1 = self.g0/(1+norm(v)^2/self.Esatw);
            v = v.*exp(L*h/2 + g1*(h/2)*K);

            % Nonlinear step
            if bitand(self.method_variant,4)
                v = fft(self.runge_kutta(h,ifft(v),fba));
            else
                v = fft(self.quadratic_exact(h,ifft(v),fba));
            end

            g1 = self.g0/(1+norm(v)^2/self.Esatw);
            v = v.*exp(L*h/2 + g1*(h/2)*K);
        end
        
        function [v,hstiff,rerror] = solver_singlestep(self,v,h,fba)
            hstiff = h;
            v1 = self.singlestep_1o(v,h,fba);

            v2 = self.singlestep_1o(v,h/2,fba);
            v2 = self.singlestep_1o(v2,h/2,fba);
            
            v = 2*v2-v1;
            rerror = norm(v-v2)/norm(v);
        end

        function vsol = runge_kutta(self,h,v,fba)
            abs_v_2 = abs(v).^2;
            k1 = 0.5*h*(self.deltagamma*abs_v_2 - self.sigma*abs_v_2.^2 - fba).*v;

            abs_v_2 = abs(v+k1).^2;
            vsol = v + h*(self.deltagamma*abs_v_2 - self.sigma*abs_v_2.^2 - fba).*(v+k1);
        end

        function vsol = quadratic_exact(self,h,v,~)
            vsol = v.*exp(-self.deltagamma/(2*real(self.deltagamma))*log(1-2*real(self.deltagamma)*h*abs(v).^2));
        end


        function err=test_N_step(self,v0,h)
            % Check the RK and exact formula match
            v1_rk = self.runge_kutta(h,v0,0);
            v1_qe = self.quadratic_exact(h,v0,0);
            
            err = norm(v1_rk-v1_qe)/norm(v0);
            fprintf('Error in N-step solve: %.5g\n',err);
        end

    end % Methods

end % Classdef
